@@ -10,6 +10,8 @@
 
 use crate::*;
 
+use nat::*;
+
 /// Core axiom of Path Semantics.
 pub type PSem<F1, F2, X1, X2> = Imply<
     And<And<Eq<F1, F2>, POrdProof<F1, X1>>,
@@ -133,24 +135,71 @@ impl<T, U> PBinOrd for POrdProof<T, U> {
 /// Path semantical proposition level.
 pub trait LProp: Prop {
     /// The level.
-    type N;
+    type N: Clone;
+    /// Sets proposition level.
+    type SetLevel<T: 'static + Clone>: LProp;
 }
 /// True for a path semantical level.
 #[derive(Copy, Clone)]
 pub struct LTrue<N>(pub N);
-impl<U: 'static + Clone> LProp for LTrue<U> {type N = U;}
-impl LProp for False {type N = nat::NaN;}
+impl<U: 'static + Clone> LProp for LTrue<U> {
+    type N = U;
+    type SetLevel<T: 'static + Clone> = LTrue<T>;
+}
+impl LProp for False {
+    type N = NaN;
+    type SetLevel<T: 'static + Clone> = Self;
+}
+/// Increases proposition level of `A` with some amount `N`.
+pub type IncLevel<A, N> = <A as LProp>::SetLevel<<(<A as LProp>::N, N) as Add>::Out>;
 
 impl<N: 'static + Default + Clone> Decidable for LTrue<N> {
     fn decide() -> ExcM<Self> {Either::Left(LTrue(N::default()))}
 }
 
-impl<T, U> POrd<U> for T where T: LProp, U: LProp, T::N: nat::Lt<U::N> {}
+impl<N: Default> Default for LTrue<N> {
+    fn default() -> Self {LTrue(N::default())}
+}
 
-/// Shorthand for decidable proposition.
+impl<T, U> POrd<U> for T where T: LProp, U: LProp, T::N: Lt<U::N> {}
+
+/// Shorthand for decidable proposition with path semantical level.
 pub trait DLProp: LProp + DProp {}
 impl<T: LProp + DProp> DLProp for T {}
 
+/// Returns the minimum LProp.
+pub type Min<A, B> = <(<A as LProp>::N, <B as LProp>::N) as SortMin<A, B>>::Out;
+/// Returns the maximum LProp.
+pub type Max<A, B> = <(<A as LProp>::N, <B as LProp>::N) as SortMax<A, B>>::Out;
+/// Normalise 4 `LProp`s (sorted ascending by propositional level).
+pub type Normalise<A, B, C, D> = (
+    Min<Min<A, B>, Min<C, D>>,
+    Min<Max<Min<A, B>, Min<C, D>>, Min<Max<A, B>, Max<C, D>>>,
+    Max<Max<Min<A, B>, Min<C, D>>, Min<Max<A, B>, Max<C, D>>>,
+    Max<Max<A, B>, Max<C, D>>,
+);
+
+/// Look by index.
+pub trait Lookup<N> {
+    /// The output type.
+    type Out;
+}
+impl<A, B, C, D> Lookup<Zero> for (A, B, C, D) {type Out = A;}
+impl<A, B, C, D> Lookup<One> for (A, B, C, D) {type Out = B;}
+impl<A, B, C, D> Lookup<Two> for (A, B, C, D) {type Out = C;}
+impl<A, B, C, D> Lookup<Three> for (A, B, C, D) {type Out = D;}
+
+/// Look up type `N` among the normalised `A, B, C, D`.
+pub type LN<N, A, B, C, D> = <Normalise<A, B, C, D> as Lookup<N>>::Out;
+
+/// Normalised naive core axiom.
+pub type PSemNaiveNorm<A, B, C, D> = PSemNaive<
+    LN<Zero, A, B, C, D>,
+    LN<One, A, B, C, D>,
+    LN<Two, A, B, C, D>,
+    LN<Three, A, B, C, D>
+>;
+
 /// Assumes the core axiom for propositions.
 pub unsafe fn assume<A: Prop, B: Prop, C: Prop, D: Prop>() -> PSem<A, B, C, D> {
     unimplemented!()
@@ -160,7 +209,7 @@ pub unsafe fn assume<A: Prop, B: Prop, C: Prop, D: Prop>() -> PSem<A, B, C, D> {
 pub fn path_level<A: LProp, B: Prop, C: LProp, D: Prop>(
     p: PSem<A, B, C, D>
 ) -> PSemNaive<A, B, C, D>
-    where A::N: nat::Lt<C::N>
+    where A::N: Lt<C::N>
 {
     Rc::new(move |(f, (a, b))| p(((f, POrdProof::new()), (a, b))))
 }
@@ -170,11 +219,125 @@ pub fn path_level<A: LProp, B: Prop, C: LProp, D: Prop>(
 /// This is safe because path semantical propositions uses the semantics
 /// that the core axiom holds between layers of propositions.
 pub fn assume_path_level<A: LProp, B: Prop, C: LProp, D: Prop>() -> PSemNaive<A, B, C, D>
-    where A::N: nat::Lt<C::N>
+    where A::N: Lt<C::N>
 {
     path_level(unsafe {assume()})
 }
 
+/// Generates naive core axiom at increased path semantical proposition level.
+pub fn assume_inc_path_level<N: Nat, A: LProp, B: LProp, C: LProp, D: LProp>()
+-> PSemNaive<IncLevel<A, N>, IncLevel<B, N>, IncLevel<C, N>, IncLevel<D, N>>
+    where <IncLevel<A, N> as LProp>::N: Lt<<IncLevel<C, N> as LProp>::N>,
+          (A::N, N): Add,
+          (B::N, N): Add,
+          (C::N, N): Add,
+          (D::N, N): Add,
+{
+    assume_path_level()
+}
+
+/// Sorts two types.
+pub trait SortMin<T: LProp, U: LProp> {
+    /// The output type.
+    type Out: LProp;
+}
+
+impl<T: LProp, U: LProp> SortMin<T, U> for (Z, Z) {
+    type Out = T;
+}
+impl<T: LProp, U: LProp, N> SortMin<T, U> for (Z, S<N>) {
+    type Out = T;
+}
+impl<T: LProp, U: LProp, N> SortMin<T, U> for (S<N>, Z) {
+    type Out = U;
+}
+impl<T: LProp, U: LProp, N, M> SortMin<T, U> for (S<N>, S<M>)
+    where (N, M): SortMin<T, U>
+{
+    type Out = <(N, M) as SortMin<T, U>>::Out;
+}
+
+/// Sorts two types.
+pub trait SortMax<T: LProp, U: LProp> {
+    /// The output type.
+    type Out: LProp;
+}
+
+impl<T: LProp, U: LProp> SortMax<T, U> for (Z, Z) {
+    type Out = U;
+}
+impl<T: LProp, U: LProp, N> SortMax<T, U> for (Z, S<N>) {
+    type Out = U;
+}
+impl<T: LProp, U: LProp, N> SortMax<T, U> for (S<N>, Z) {
+    type Out = T;
+}
+impl<T: LProp, U: LProp, N, M> SortMax<T, U> for (S<N>, S<M>)
+    where (N, M): SortMax<T, U>
+{
+    type Out = <(N, M) as SortMax<T, U>>::Out;
+}
+
+#[cfg(test)]
+pub mod tests {
+    use super::*;
+
+    fn check_sort_min<X, Y, U: LProp, T: LProp>() where (X, Y): SortMin<U, T> {}
+
+    pub fn sort_min<T: LProp, U: LProp>() {
+        check_sort_min::<Z, Z, T, U>();
+        check_sort_min::<S<Z>, Z, T, U>();
+        check_sort_min::<Z, S<Z>, T, U>();
+        check_sort_min::<S<Z>, S<Z>, T, U>();
+    }
+}
+
+/// The decided minimum (1st of 4).
+pub type MinMin<A, B, C, D> = Min<Min<A, B>, Min<C, D>>;
+/// The undecided maximum minimum.
+pub type MaxMin<A, B, C, D> = Max<Min<A, B>, Min<C, D>>;
+/// The undecided minimum maximum.
+pub type MinMax<A, B, C, D> = Min<Max<A, B>, Max<C, D>>;
+/// The decided maximum (4th of 4).
+pub type MaxMax<A, B, C, D> = Max<Max<A, B>, Max<C, D>>;
+/// The decided minimum of undecided middle (2nd of 4).
+pub type Mixi<A, B, C, D> = Min<MaxMin<A, B, C, D>, MinMax<A, B, C, D>>;
+/// The decided maximum of undecided middle (3rd of 4).
+pub type Maxi<A, B, C, D> = Max<MaxMin<A, B, C, D>, MinMax<A, B, C, D>>;
+
+/// Assume a normalised naive core axiom.
+///
+/// The orientation is detected automatically and restored to a naive core axiom
+/// which has a proof to any valid order.
+pub fn assume_norm_path_level<A: LProp, B: LProp, C: LProp, D: LProp>()
+-> PSemNaiveNorm<A, B, C, D>
+    where
+        (A::N, B::N):
+            SortMin<A, B> +
+            SortMax<A, B>,
+        (C::N, D::N): SortMin<C, D> + SortMax<C, D>,
+        (<Min<A, B> as LProp>::N, <Min<C, D> as LProp>::N):
+            SortMin<Min<A, B>, Min<C, D>> +
+            SortMax<Min<A, B>, Min<C, D>>,
+        (<Max<A, B> as LProp>::N, <Max<C, D> as LProp>::N):
+            SortMin<Max<A, B>, Max<C, D>> +
+            SortMax<Max<A, B>, Max<C, D>>,
+        (<MaxMin<A, B, C, D> as LProp>::N, <MinMax<A, B, C, D> as LProp>::N):
+            SortMin<MaxMin<A, B, C, D>, MinMax<A, B, C, D>> +
+            SortMax<MaxMin<A, B, C, D>, MinMax<A, B, C, D>>,
+        <MinMin<A, B, C, D> as LProp>::N:
+            Lt<<Maxi<A, B, C, D> as LProp>::N>,
+{
+    assume_path_level()
+}
+
+/// Generates a naive core axiom which has reflection as end-lines.
+pub fn assume_path_refl<A: LProp, B: LProp>() -> PSemNaive<A, A, B, B>
+    where A::N: Lt<B::N>
+{
+    assume_path_level()
+}
+
 /// Reduce core axiom in case of false to equality of associated propositions.
 pub fn red_false<A: Prop, B: Prop>(
     p: PSem<False, False, A, B>
@@ -207,6 +370,22 @@ pub fn comp<F1: Prop, F2: Prop, F3: Prop, F4: Prop, X1: Prop, X2: Prop>(
     })
 }
 
+/// Composition using the naive core axiom.
+pub fn naive_comp<F1: Prop, F2: Prop, F3: Prop, F4: Prop, X1: Prop, X2: Prop>(
+    f: PSemNaive<F1, F2, F3, F4>,
+    g: PSemNaive<F3, F4, X1, X2>,
+    f1_f3: Imply<F1, F3>,
+    f2_f4: Imply<F2, F4>,
+    f3_x1: Imply<F3, X1>,
+    f4_x2: Imply<F4, X2>,
+) -> PSemNaive<F1, F2, X1, X2> {
+    Rc::new(move |(f1_eq_f2, (_f1_x1, _f2_x2))| {
+        let f3_eq_f4 = f((f1_eq_f2, (f1_f3.clone(), f2_f4.clone())));
+        let x1_eq_x2 = g((f3_eq_f4, (f3_x1.clone(), f4_x2.clone())));
+        x1_eq_x2
+    })
+}
+
 /// Converts core axiom to `PAndFst`.
 pub fn to_pand_fst<A: Prop, B: Prop, C: Prop, D: Prop>(
     p: PSem<And<A, B>, C, A, D>
@@ -288,15 +467,14 @@ pub fn uniq_ty<A: Prop, B: Prop, C: Prop, D: Prop, E: Prop>(
 }
 
 /// Checks whether two proposition levels are equal.
-pub fn eq_lev<A: LProp, B: LProp>(_a: A, _b: B) where (A::N, B::N): nat::EqNat {}
+pub fn eq_lev<A: LProp, B: LProp>(_a: A, _b: B) where (A::N, B::N): EqNat {}
 /// Checks whether a proposition level is less than another.
-pub fn lt_lev<A: LProp, B: LProp>(_a: A, _b: B) where A::N: nat::Lt<B::N> {}
+pub fn lt_lev<A: LProp, B: LProp>(_a: A, _b: B) where A::N: Lt<B::N> {}
 
 #[cfg(test)]
 #[allow(dead_code)]
 mod test {
     use super::*;
-    use nat::*;
 
     fn check_nan<A: LProp<N = NaN>, B: LProp<N = NaN>>(a: A, b: B) {eq_lev(a, b)}
     fn check_zero<A: LProp<N = Zero>, B: LProp<N = Zero>>(a: A, b: B) {eq_lev(a, b)}
@@ -307,4 +485,5 @@ mod test {
     {
         eq_lev(a, b)
     }
+    fn check_one_two() {lt_lev(LTrue(_1), LTrue(_2))}
 }
